/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include <sstream>
#include <iostream>

using namespace std;
using namespace FD;

class CvfdhSaveImage;
   
DECLARE_NODE(CvfdhSaveImage)
/*Node
 *
 * @name CvfdhSaveImage
 * @category FDOPENCV_HARK:ImageIO
 * @description View an image
 *
 * @input_name IMAGEIN
 * @input_description Image to visualize
 * @input_type CvfdImage
 *
 * @parameter_name FILENAME_PREFIX
 * @parameter_type string
 * @parameter_value img
 * @parameter_description Prefix of the saved file name
 *
 * @output_name IMAGEOUT
 * @output_description Same as input image 
 * @output_type CvfdImage
 *
 END*/
   
   
class CvfdhSaveImage : public BufferedNode {
      
  int m_imageInID;
  int m_imageOutID;
  string filename_prefix;
            
public:
  CvfdhSaveImage(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    m_imageOutID = addOutput("IMAGEOUT");         
    filename_prefix = object_cast<String>(parameters.get("FILENAME_PREFIX"));
    inOrder = true;
  }
      
  ~CvfdhSaveImage(){}
      
  void calculate(int output_id, int count, Buffer &out)
  {
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
    if(imagePtr->getImage() == NULL){
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);
      return;
    }

    ostringstream os;
    os << filename_prefix << "_" << count << ".jpg";
    cout << os.str() << endl;
         
    cvSaveImage (os.str().c_str(), imagePtr->getImage());

    (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhSaveImage)
};
